package logic

import (
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/validate"
	"time"

	"context"

	"fusenapi/server/home-user-auth/internal/svc"
	"fusenapi/server/home-user-auth/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type UserInfoSetLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUserInfoSetLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserInfoSetLogic {
	return &UserInfoSetLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UserInfoSetLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UserInfoSetLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *UserInfoSetLogic) UserInfoSet(req *types.UserInfoSetReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	if userinfo.IsOnlooker() {
		// 如果是，返回未授权的错误码
		return resp.SetStatus(basic.CodeUnAuth)
	}

	var userId int64
	var guestId int64

	// 检查用户是否是游客
	if userinfo.IsGuest() {
		// 如果是，使用游客ID和游客键名格式
		guestId = userinfo.GuestId
	} else {
		// 否则，使用用户ID和用户键名格式
		userId = userinfo.UserId
	}

	_, err := validate.Validate(&req.Module, &req.Metadata)
	if err != nil {
		logx.Error(err)
		return resp.SetStatus(basic.CodeRequestParamsErr, "UserInfoSet error Validate failed")
	}

	// 根据模块类型检查数据
	var userInfo = &gmodel.FsUserInfo{}
	fsUserInfoModel := gmodel.NewFsUserInfoModel(l.svcCtx.MysqlConn)
	BuilderDB := fsUserInfoModel.BuilderDB(l.ctx, nil).Model(&gmodel.FsUserInfo{})
	BuilderDB1 := BuilderDB.Where("module = ?", req.Module).Where("user_id=?", userId).Where("guest_id=?", guestId)
	userInfo, err = fsUserInfoModel.FindOne(BuilderDB1, nil)
	if err != nil {
		logx.Error(err)
		return resp.SetStatus(basic.CodeDbSqlErr, "UserInfoSet error system failed")
	}
	var nowTime = time.Now().Unix()
	if userInfo.Id != 0 {
		userInfo.Metadata = &req.Metadata
		userInfo.Utime = &nowTime
	} else {
		userInfo.GuestId = &guestId
		userInfo.UserId = &userId
		userInfo.Module = &req.Module
		userInfo.Metadata = &req.Metadata
		userInfo.Ctime = &nowTime
	}
	_, err = fsUserInfoModel.CreateOrUpdate(BuilderDB, userInfo)
	if err != nil {
		logx.Error(err)
		return resp.SetStatus(basic.CodeDbSqlErr, "UserInfoSet error system failed")
	}

	return resp.SetStatus(basic.CodeOK)
}
