package logic

import (
	"context"
	"encoding/json"
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"gorm.io/gorm"

	"fusenapi/server/product-template-tag/internal/svc"
	"fusenapi/server/product-template-tag/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetProductTemplateTagsLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetProductTemplateTagsLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetProductTemplateTagsLogic {
	return &GetProductTemplateTagsLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *GetProductTemplateTagsLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *GetProductTemplateTagsLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *GetProductTemplateTagsLogic) GetProductTemplateTags(req *types.GetProductTemplateTagsReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.Limit <= 0 || req.Limit > 100 {
		req.Limit = 5
	}
	var (
		productTemplateTags []gmodel.FsProductTemplateTags
	)
	//获取用户元数据
	userMaterial, err := l.svcCtx.AllModels.FsUserMaterial.GetListByUser(l.ctx, userinfo.UserId, userinfo.GuestId, req.Limit)
	if err != nil {
		if !errors.Is(err, gorm.ErrRecordNotFound) {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get ai recommend product template tag list")
		}
		// 返回固定模板标签列表
		productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetList(l.ctx, 1, req.Limit, "`id` DESC")
	} else {
		//元数据是空的
		if userMaterial.Metadata == nil || *userMaterial.Metadata == "" {
			// 返回固定模板标签列表
			productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetList(l.ctx, 1, req.Limit, "`id` DESC")
		} else {
			//解析元数据
			var metaData map[string]interface{}
			if err = json.Unmarshal([]byte(*userMaterial.Metadata), &metaData); err != nil {
				logx.Error(err)
				return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user metadata")
			}
			var templateTagNameList []string
			b, _ := json.Marshal(metaData["template_tagid"])
			if err = json.Unmarshal(b, &templateTagNameList); err != nil {
				logx.Error(err)
				return resp.SetStatusWithMessage(basic.CodeJsonErr, "invalid format of metadata`s template_tagid")
			}
			productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetListByTagNames(l.ctx, templateTagNameList, req.Limit, "id DESC")
		}
	}
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
	}
	list := make([]types.GetProductTemplateTagsRsp, 0, len(productTemplateTags))
	for _, v := range productTemplateTags {
		list = append(list, types.GetProductTemplateTagsRsp{
			Id:          v.Id,
			TemplateTag: *v.TemplateTag,
			Cover:       *v.Cover,
		})
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", list)
}
