package logic

import (
	"context"
	"fusenapi/server/websocket/internal/svc"
	"fusenapi/server/websocket/internal/types"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/file"
	"time"

	"github.com/zeromicro/go-zero/core/logx"
)

type RenderNotifyLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewRenderNotifyLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RenderNotifyLogic {
	return &RenderNotifyLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *RenderNotifyLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *RenderNotifyLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *RenderNotifyLogic) RenderNotify(req *types.RenderNotifyReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.TaskId == "" {
		logx.Error("渲染回调参数错误：invalid param task_id")
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid param task_id")
	}
	if req.Image == "" {
		logx.Error("渲染回调参数错误：invalid param image")
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "invalid param image")
	}
	//存base64打印测试
	/*	f, _ := os.Create("b.txt")
		defer f.Close()
		f.WriteString(req.Image)*/
	// 上传文件
	var upload = file.Upload{
		Ctx:        l.ctx,
		MysqlConn:  l.svcCtx.MysqlConn,
		AwsSession: l.svcCtx.AwsSession,
	}
	uploadRes, err := upload.UploadFileByBase64(&file.UploadBaseReq{
		Source:       "unity cloud render",
		FileHash:     req.TaskId,
		FileData:     req.Image,
		Metadata:     "",
		UploadBucket: 1,
		ApiType:      2,
		UserId:       req.UserId,
		GuestId:      req.GuestId,
		FileByte:     nil,
	})
	if err != nil {
		logx.Error("渲染回调上传文件失败：", err)
		return resp.SetStatusWithMessage(basic.CodeFileUploadErr, "failed to upload render resource image")
	}
	//遍历websocket链接把数据传进去
	mapConnPool.Range(func(key, value any) bool {
		//断言连接
		ws, ok := value.(wsConnectItem)
		if !ok {
			logx.Error("渲染回调断言websocket连接失败")
			return true
		}
		//记录收到unity渲染结果时间
		ws.modifyRenderTaskTimeConsuming(renderImageControlChanItem{
			Option: 2,
			TaskProperty: renderTask{
				UnityRenderEndTime: time.Now().UTC().Unix(),
			},
		})
		//发送处理并删除任务
		ws.deleteRenderTask(renderImageControlChanItem{
			Option:               0, //0删除 1添加
			TaskId:               req.TaskId,
			RenderNotifyImageUrl: uploadRes.ResourceUrl,
		})
		return true
	})
	logx.Info("渲染回调成功,渲染结果图片为：", uploadRes.ResourceUrl)
	return resp.SetStatusWithMessage(basic.CodeOK, "success")
}
